@@ -22,6 +22,7 @@
 #define IRCSERVERHANDLER_H
 
 #include "basichandler.h"
+#include "netsplit.h"
 
 class IrcServerHandler : public BasicHandler {
   Q_OBJECT
@@ -68,6 +69,7 @@ public slots:
   void handle322(const QString &prefix, const QList<QByteArray> &params);   // RPL_LIST
   void handle323(const QString &prefix, const QList<QByteArray> &params);   // RPL_LISTEND
   void handle324(const QString &prefix, const QList<QByteArray> &params);   // RPL_CHANNELMODEIS
+  void handle328(const QString &prefix, const QList<QByteArray> &params);   // RPL_??? (channel homepage)
   void handle329(const QString &prefix, const QList<QByteArray> &params);   // RPL_??? (channel creation time)
   void handle331(const QString &prefix, const QList<QByteArray> &params);   // RPL_NOTOPIC
   void handle332(const QString &prefix, const QList<QByteArray> &params);   // RPL_TOPIC
@@ -80,6 +82,31 @@ public slots:
 
   void defaultHandler(QString cmd, const QString &prefix, const QList<QByteArray> &params);
 
+private slots:
+  //! Joins after a netsplit
+  /** This slot handles a bulk-join after a netsplit is over
+    * \param channel The channel the users joined
+    * \param users   The list of users that joind the channel
+    * \param modes   The list of modes the users get set
+    * \param quitMessage The message we received when the netsplit occured
+    */
+  void handleNetsplitJoin(const QString &channel, const QStringList &users, const QStringList &modes, const QString &quitMessage);
+
+  //! Quits after a netsplit
+  /** This slot handles a bulk-quit after a netsplit occured
+    * \param channel The channel the users quitted
+    * \param users   The list of users that got split
+    * \param quitMessage The message we received when the netsplit occured
+    */
+  void handleNetsplitQuit(const QString &channel, const QStringList &users, const QString &quitMessage);
+
+  //! Netsplit finished
+  /** This slot deletes the netsplit object that sent the finished() signal
+    */
+  void handleNetsplitFinished();
+
+  void handleEarlyNetsplitJoin(const QString &channel, const QStringList &users, const QStringList &modes);
+
 private:
   void tryNextNick(const QString &errnick, bool erroneus = false);
   bool checkParamCount(const QString &methodName, const QList<QByteArray> &params, int minParams);
@@ -89,6 +116,11 @@ private:
 
   bool _whois;
   QString _target;
+
+  // structure to organize netsplits
+  // key: quit message
+  // value: the corresponding netsplit object
+  QHash<QString, Netsplit*> _netsplits;
 };
 
 
