@@ -79,32 +79,44 @@ QString AboutDlg::contributors() const {
            "<dl>"
            "<dt><b>Daniel \"al\" Albers</b></dt><dd>German translation, various fixes and enhancements</dd>"
            "<dt><b>Terje \"tan\" Andersen</b></dt><dd>Norwegian translation, documentation</dd>"
-           "<dt><b>Tomas \"scarabeus\" Chvatal</b></dt><dd>Czech translation</dd>"
+           "<dt><b>Tomáš \"scarabeus\" Chvátal</b></dt><dd>Czech translation</dd>"
            "<dt><b>Joshua \"tvakah\" Corbin</b></dt><dd>Various fixes</dd>"
+           "<dt><b>Matthias \"pennywise\" Coy</b></dt><dd>German translation</dd>"
            "<dt><b>Kevin \"KRF\" Funk</b></dt><dd>German translation</dd>"
            "<dt><b>Marco \"kaffeedoktor\" Genise</b></dt><dd>Ideas, hacking, motivation</dd>"
            "<dt><b>Sebastian \"seezer\" Goth</b></dt><dd>Various improvements and features</dd>"
+           "<dt><b>Michael Groh</b></dt><dd>German translation</dd>"
            "<dt><b>H. İbrahim \"igungor\" Güngör</b></dt><dd>Turkish translation</dd>"
            "<dt><b>Edward \"Aides\" Toroshchin</b></dt><dd>Russian translation</dd>"
            "<dt><b>Fabiano \"elbryan\" Francesconi</b></dt><dd>Italian translation</dd>"
            "<dt><b>John \"nox\" Hand</b></dt><dd>Former All-Seeing Eye logo</dd>"
            "<dt><b>Jonas \"Dante\" Heese</b></dt><dd>Project founder, various improvements</dd>"
+           "<dt><b>Aurélien \"agateau\" Gâteau</b></dt><dd>Message Indicator support</dd>"
            "<dt><b>Scott \"ScottK\" Kitterman<b></dt><dd>Kubuntu nightly packager, (packaging/build system) bughunter</dd>"
            "<dt><b>Paul \"Haudrauf\" Klumpp</b></dt><dd>Initial design and mainwindow layout</dd>"
+           "<dt><b>Maia Kozheva</b></dt><dd>Russian translation</dd>"
+           "<dt><b>Martin \"m4yer\" Mayer</b></dt><dd>German translation</dd>"
            "<dt><b>Gábor \"ELITE_x\" Németh</b></dt><dd>Hungarian translation</dd>"
+           "<dt><b>Marco \"Quizzlo\" Paolone</b></dt><dd>Italian translation</dd>"
+           "<dt><b>Drew \"LinuxDolt\" Patridge</b></dt><dd>BluesTheme stylesheet</dd>"
            "<dt><b>Celeste \"seele\" Paul</b></dt><dd>Usability Queen</dd>"
            "<dt><b>Regis \"ZRegis\" Perrin</b></dt><dd>French translation</dd>"
            "<dt><b>Diego \"Flameeyes\" Petten&ograve;</b></dt><dd>Gentoo maintainer, build system improvements</dd>"
            "<dt><b>Daniel \"billie\" Pielmeier</b></dt><dd>Gentoo maintainer</dd>"
            "<dt><b>Nuno \"pinheiro\" Pinheiro</b></dt><dd>Tons of Oxygen icons including our application icon</dd>"
            "<dt><b>Jure \"JLP\" Repinc</b></dt><dd>Slovenian translation</dd>"
+           "<dt><b>Henning \"honk\" Rohlfs</b></dt><dd>Various fixes</dd>"
            "<dt><b>Dennis \"DevUrandom\" Schridde</b></dt><dd>D-Bus notifications</dd>"
            "<dt><b>Jussi \"jussi01\" Schultink</b></dt><dd>Tireless tester, {ku|U}buntu tester and lobbyist, liters of delicious Finnish alcohol</dd>"
            "<dt><b>Tim \"xAFFE\" Schumacher</b></dt><dd>Fixes and feedback</dd>"
            "<dt><b>Harald \"apachelogger\" Sitter</b></dt><dd>{ku|U}buntu packager, motivator, promoter</dd>"
            "<dt><b>Daniel \"son\" Steinmetz</b></dt><dd>Early beta tester and bughunter (on Vista&trade;!)</dd>"
+           "<dt><b>Jesper Thomschütz</b></dt><dd>Various fixes</dd>"
            "<dt><b>Adam \"adamt\" Tulinius</b></dt><dd>Early beta tester and bughunter, Danish translation</dd>"
+           "<dt><b>Frederik M.J. Vestre</b></dt><dd>Norwegian translation</dd>"
+           "<dt><b>Atte Virtanen</b></dt><dd>Finnish translation</dd>"
            "<dt><b>Pavel \"int\" Volkovitskiy</b></dt><dd>Early beta tester and bughunter</dd>"
+           "<dt><b>Benjamin \"zbenjamin\" Zeller</b></dt><dd>Windows build system fixes</dd>"
            "</dl><br>"
            "...and anybody else finding and reporting bugs, giving feedback, helping others and being part of the community!");
 
