@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-08 by the Quassel IRC Team                         *
+ *   Copyright (C) 2005-09 by the Quassel Project                          *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,25 +18,47 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include "coreconnectdlg.h"
+
 #include <QDebug>
+#include <QFormLayout>
 #include <QMessageBox>
 #include <QNetworkProxy>
 
-#include "coreconnectdlg.h"
+#ifdef HAVE_SSL
+#include <QSslSocket>
+#include <QSslCertificate>
+#endif
 
+#include "client.h"
 #include "clientsettings.h"
 #include "clientsyncer.h"
 #include "coreconfigwizard.h"
+#include "iconloader.h"
+#include "quassel.h"
+#include "util.h"
 
 CoreConnectDlg::CoreConnectDlg(bool autoconnect, QWidget *parent)
-  : QDialog(parent)
+  : QDialog(parent),
+    _internalAccountId(0)
 {
   ui.setupUi(this);
+  ui.editAccount->setIcon(SmallIcon("document-properties"));
+  ui.addAccount->setIcon(SmallIcon("list-add"));
+  ui.deleteAccount->setIcon(SmallIcon("list-remove"));
+  ui.connectIcon->setPixmap(BarIcon("network-disconnect"));
+  ui.secureConnection->setPixmap(SmallIcon("document-encrypt"));
+
+  if(Quassel::runMode() != Quassel::Monolithic) {
+    ui.useInternalCore->hide();
+  }
 
   // make it look more native under Mac OS X:
   setWindowFlags(Qt::Sheet);
 
   clientSyncer = new ClientSyncer(this);
+  Client::registerClientSyncer(clientSyncer);
+
   wizard = 0;
 
   doingAutoConnect = false;
@@ -50,6 +72,10 @@ CoreConnectDlg::CoreConnectDlg(bool autoconnect, QWidget *parent)
   foreach(AccountId id, s.knownAccounts()) {
     if(!id.isValid()) continue;
     QVariantMap data = s.retrieveAccountData(id);
+    if(data.contains("InternalAccount") && data["InternalAccount"].toBool()) {
+      _internalAccountId = id;
+      continue;
+    }
     data["AccountId"] = QVariant::fromValue<AccountId>(id);
     accounts[id] = data;
     QListWidgetItem *item = new QListWidgetItem(data["AccountName"].toString(), ui.accountList);
@@ -66,8 +92,8 @@ CoreConnectDlg::CoreConnectDlg(bool autoconnect, QWidget *parent)
 
   connect(clientSyncer, SIGNAL(socketStateChanged(QAbstractSocket::SocketState)),this, SLOT(initPhaseSocketState(QAbstractSocket::SocketState)));
   connect(clientSyncer, SIGNAL(connectionError(const QString &)), this, SLOT(initPhaseError(const QString &)));
+  connect(clientSyncer, SIGNAL(connectionWarnings(const QStringList &)), this, SLOT(initPhaseWarnings(const QStringList &)));
   connect(clientSyncer, SIGNAL(connectionMsg(const QString &)), this, SLOT(initPhaseMsg(const QString &)));
-  connect(clientSyncer, SIGNAL(encrypted(bool)), this, SLOT(encrypted(bool)));
   connect(clientSyncer, SIGNAL(startLogin()), this, SLOT(startLogin()));
   connect(clientSyncer, SIGNAL(loginFailed(const QString &)), this, SLOT(loginFailed(const QString &)));
   connect(clientSyncer, SIGNAL(loginSuccess()), this, SLOT(startSync()));
@@ -75,6 +101,7 @@ CoreConnectDlg::CoreConnectDlg(bool autoconnect, QWidget *parent)
   connect(clientSyncer, SIGNAL(sessionProgress(quint32, quint32)), this, SLOT(coreSessionProgress(quint32, quint32)));
   connect(clientSyncer, SIGNAL(networksProgress(quint32, quint32)), this, SLOT(coreNetworksProgress(quint32, quint32)));
   connect(clientSyncer, SIGNAL(syncFinished()), this, SLOT(syncFinished()));
+  connect(clientSyncer, SIGNAL(encrypted()), ui.secureConnection, SLOT(show()));
 
   connect(ui.user, SIGNAL(textChanged(const QString &)), this, SLOT(setLoginWidgetStates()));
   connect(ui.password, SIGNAL(textChanged(const QString &)), this, SLOT(setLoginWidgetStates()));
@@ -135,14 +162,7 @@ void CoreConnectDlg::on_addAccount_clicked() {
   for(int i = 0; i < ui.accountList->count(); i++) existing << ui.accountList->item(i)->text();
   CoreAccountEditDlg dlg(0, QVariantMap(), existing, this);
   if(dlg.exec() == QDialog::Accepted) {
-    // find free ID
-    AccountId id = accounts.count() + 1;
-    for(AccountId i = 1; i <= accounts.count(); i++) {
-      if(!accounts.keys().contains(i)) {
-        id = i;
-        break;
-      }
-    }
+    AccountId id = findFreeAccountId();
     QVariantMap data = dlg.accountData();
     data["AccountId"] = QVariant::fromValue<AccountId>(id);
     accounts[id] = data;
@@ -205,6 +225,11 @@ void CoreConnectDlg::on_accountButtonBox_accepted() {
   connectToCore();
 }
 
+void CoreConnectDlg::on_useInternalCore_clicked() {
+  clientSyncer->useInternalCore();
+  ui.loginButtonBox->setStandardButtons(QDialogButtonBox::Cancel);
+}
+
 /*****************************************************
  * Connecting to the Core
  ****************************************************/
@@ -213,7 +238,7 @@ void CoreConnectDlg::on_accountButtonBox_accepted() {
 
 void CoreConnectDlg::connectToCore() {
   ui.secureConnection->hide();
-  ui.connectIcon->setPixmap(QPixmap::fromImage(QImage(":/22x22/actions/network-disconnect")));
+  ui.connectIcon->setPixmap(BarIcon("network-disconnect"));
   ui.connectLabel->setText(tr("Connect to %1").arg(accountData["Host"].toString()));
   ui.coreInfoLabel->setText("");
   ui.loginStack->setCurrentWidget(ui.loginEmptyPage);
@@ -229,7 +254,7 @@ void CoreConnectDlg::connectToCore() {
 void CoreConnectDlg::initPhaseError(const QString &error) {
   doingAutoConnect = false;
   ui.secureConnection->hide();
-  ui.connectIcon->setPixmap(QPixmap::fromImage(QImage(":/22x22/status/dialog-error")));
+  ui.connectIcon->setPixmap(BarIcon("dialog-error"));
   //ui.connectLabel->setBrush(QBrush("red"));
   ui.connectLabel->setText(tr("<div style=color:red;>Connection to %1 failed!</div>").arg(accountData["Host"].toString()));
   ui.coreInfoLabel->setText(error);
@@ -240,15 +265,41 @@ void CoreConnectDlg::initPhaseError(const QString &error) {
   connect(ui.loginButtonBox, SIGNAL(rejected()), this, SLOT(reject()));
 }
 
-void CoreConnectDlg::initPhaseMsg(const QString &msg) {
-  ui.coreInfoLabel->setText(msg);
+void CoreConnectDlg::initPhaseWarnings(const QStringList &warnings) {
+  doingAutoConnect = false;
+  ui.secureConnection->hide();
+  ui.connectIcon->setPixmap(BarIcon("dialog-warning"));
+  ui.connectLabel->setText(tr("<div>Errors occurred while connecting to \"%1\":</div>").arg(accountData["Host"].toString()));
+  QStringList warningItems;
+  foreach(QString warning, warnings) {
+    warningItems << QString("<li>%1</li>").arg(warning);
+  }
+  ui.coreInfoLabel->setText(QString("<ul>%1</ul>").arg(warningItems.join("")));
+  ui.loginStack->setCurrentWidget(ui.connectionWarningsPage);
+  ui.loginButtonBox->setStandardButtons(QDialogButtonBox::Cancel);
+  ui.loginButtonBox->button(QDialogButtonBox::Cancel)->setFocus();
+  disconnect(ui.loginButtonBox, 0, this, 0);
+  connect(ui.loginButtonBox, SIGNAL(rejected()), this, SLOT(restartPhaseNull()));
 }
 
-void CoreConnectDlg::encrypted(bool useSsl) {
-  if(useSsl)
-    ui.secureConnection->show();
-  else
-    ui.secureConnection->hide();
+void CoreConnectDlg::on_viewSslCertButton_clicked() {
+#ifdef HAVE_SSL
+  const QSslSocket *socket = qobject_cast<const QSslSocket *>(clientSyncer->currentDevice());
+  if(!socket)
+    return;
+
+  SslCertDisplayDialog dialog(socket->peerName(), socket->peerCertificate());
+  dialog.exec();
+#endif
+}
+
+void CoreConnectDlg::on_ignoreWarningsButton_clicked() {
+  clientSyncer->ignoreWarnings(ui.ignoreWarningsPermanently->isChecked());
+}
+
+
+void CoreConnectDlg::initPhaseMsg(const QString &msg) {
+  ui.coreInfoLabel->setText(msg);
 }
 
 void CoreConnectDlg::initPhaseSocketState(QAbstractSocket::SocketState state) {
@@ -275,7 +326,7 @@ void CoreConnectDlg::restartPhaseNull() {
  *********************************************************/
 
 void CoreConnectDlg::startLogin() {
-  ui.connectIcon->setPixmap(QPixmap::fromImage(QImage(":/22x22/actions/network-connect")));
+  ui.connectIcon->setPixmap(BarIcon("network-connect"));
   ui.loginStack->setCurrentWidget(ui.loginCredentialsPage);
   //ui.loginStack->setMinimumSize(ui.loginStack->sizeHint()); ui.loginStack->updateGeometry();
   ui.loginButtonBox->setStandardButtons(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
@@ -336,6 +387,7 @@ void CoreConnectDlg::loginFailed(const QString &error) {
   if(wizard) {
     wizard->reject();
   }
+  ui.connectIcon->setPixmap(BarIcon("dialog-error"));
   ui.loginStack->setCurrentWidget(ui.loginCredentialsPage);
   ui.loginGroup->setTitle(tr("Login"));
   ui.user->setEnabled(true);
@@ -399,7 +451,8 @@ void CoreConnectDlg::startSync() {
   ui.user->setEnabled(true);
   ui.password->setEnabled(true);
   ui.rememberPasswd->setEnabled(true);
-  ui.loginButtonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
+  if(ui.loginButtonBox->standardButtons() & QDialogButtonBox::Ok) // in mono mode we don't show an Ok Button
+    ui.loginButtonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
 }
 
 void CoreConnectDlg::coreSessionProgress(quint32 val, quint32 max) {
@@ -429,6 +482,13 @@ void CoreConnectDlg::syncFinished() {
   }
 }
 
+AccountId CoreConnectDlg::findFreeAccountId() {
+  for(AccountId i = 1;; i++) {
+    if(!accounts.contains(i) && i != _internalAccountId)
+      return i;
+  }
+}
+
 /*****************************************************************************************
  * CoreAccountEditDlg
  *****************************************************************************************/
@@ -436,14 +496,17 @@ CoreAccountEditDlg::CoreAccountEditDlg(AccountId id, const QVariantMap &acct, co
   : QDialog(parent)
 {
   ui.setupUi(this);
+  ui.useSsl->setIcon(SmallIcon("document-encrypt"));
+
   existing = _existing;
   if(id.isValid()) {
+    account = acct;
+
     existing.removeAll(acct["AccountName"].toString());
     ui.host->setText(acct["Host"].toString());
     ui.port->setValue(acct["Port"].toUInt());
-    ui.useInternal->setChecked(acct["UseInternal"].toBool());
     ui.accountName->setText(acct["AccountName"].toString());
-#ifndef QT_NO_OPENSSL
+#ifdef HAVE_SSL
     ui.useSsl->setChecked(acct["useSsl"].toBool());
 #else
     ui.useSsl->setChecked(false);
@@ -457,26 +520,17 @@ CoreAccountEditDlg::CoreAccountEditDlg(AccountId id, const QVariantMap &acct, co
     ui.proxyPassword->setText(acct["proxyPassword"].toString());
   } else {
     setWindowTitle(tr("Add Core Account"));
-#ifdef QT_NO_OPENSSL
+#ifndef HAVE_SSL
     ui.useSsl->setChecked(false);
     ui.useSsl->setEnabled(false);
 #endif
   }
-
-#ifndef BUILD_MONO
-  // if we don't have a mono build we hide the option to use the internal connection and force the setting to use remote host
-  ui.useInternal->setChecked(false);
-  ui.useInternal->hide();
-  ui.useRemote->hide();
-  ui.labelUseBuiltinCore->hide();
-#endif
 }
 
 QVariantMap CoreAccountEditDlg::accountData() {
   account["AccountName"] = ui.accountName->text().trimmed();
   account["Host"] = ui.host->text().trimmed();
   account["Port"] = ui.port->value();
-  account["UseInternal"] = ui.useInternal->isChecked();
   account["useSsl"] = ui.useSsl->isChecked();
   account["useProxy"] = ui.useProxy->isChecked();
   account["proxyHost"] = ui.proxyHost->text().trimmed();
@@ -488,7 +542,7 @@ QVariantMap CoreAccountEditDlg::accountData() {
 }
 
 void CoreAccountEditDlg::setWidgetStates() {
-  bool ok = !ui.accountName->text().trimmed().isEmpty() && !existing.contains(ui.accountName->text()) && (ui.useInternal->isChecked() || !ui.host->text().isEmpty());
+  bool ok = !ui.accountName->text().trimmed().isEmpty() && !existing.contains(ui.accountName->text()) && !ui.host->text().isEmpty();
   ui.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(ok);
 }
 
@@ -502,7 +556,57 @@ void CoreAccountEditDlg::on_accountName_textChanged(const QString &text) {
   setWidgetStates();
 }
 
-void CoreAccountEditDlg::on_useRemote_toggled(bool state) {
-  Q_UNUSED(state);
-  setWidgetStates();
+
+
+// ========================================
+//  SslCertDisplayDialog
+// ========================================
+#ifdef HAVE_SSL
+SslCertDisplayDialog::SslCertDisplayDialog(const QString &host, const QSslCertificate &cert, QWidget *parent)
+  : QDialog(parent)
+{
+  setWindowTitle(tr("SSL Certificate used by %1").arg(host));
+
+  QVBoxLayout *mainLayout = new QVBoxLayout(this);
+
+  QGroupBox *issuerBox = new QGroupBox(tr("Issuer Info"), this);
+  QFormLayout *issuerLayout = new QFormLayout(issuerBox);
+  issuerLayout->addRow(tr("Organization:"), new QLabel(cert.issuerInfo(QSslCertificate::Organization), this));
+  issuerLayout->addRow(tr("Locality Name:"), new QLabel(cert.issuerInfo(QSslCertificate::LocalityName), this));
+  issuerLayout->addRow(tr("Organizational Unit Name:"), new QLabel(cert.issuerInfo(QSslCertificate::OrganizationalUnitName), this));
+  issuerLayout->addRow(tr("Country Name:"), new QLabel(cert.issuerInfo(QSslCertificate::CountryName), this));
+  issuerLayout->addRow(tr("State or Province Name:"), new QLabel(cert.issuerInfo(QSslCertificate::StateOrProvinceName), this));
+  mainLayout->addWidget(issuerBox);
+
+  QGroupBox *subjectBox = new QGroupBox(tr("Subject Info"), this);
+  QFormLayout *subjectLayout = new QFormLayout(subjectBox);
+  subjectLayout->addRow(tr("Organization:"), new QLabel(cert.subjectInfo(QSslCertificate::Organization), this));
+  subjectLayout->addRow(tr("Locality Name:"), new QLabel(cert.subjectInfo(QSslCertificate::LocalityName), this));
+  subjectLayout->addRow(tr("Organizational Unit Name:"), new QLabel(cert.subjectInfo(QSslCertificate::OrganizationalUnitName), this));
+  subjectLayout->addRow(tr("Country Name:"), new QLabel(cert.subjectInfo(QSslCertificate::CountryName), this));
+  subjectLayout->addRow(tr("State or Province Name:"), new QLabel(cert.subjectInfo(QSslCertificate::StateOrProvinceName), this));
+  mainLayout->addWidget(subjectBox);
+
+  QGroupBox *additionalBox = new QGroupBox(tr("Additional Info"), this);
+  QFormLayout *additionalLayout = new QFormLayout(additionalBox);
+  additionalLayout->addRow(tr("Valid From:"), new QLabel(cert.effectiveDate().toString(), this));
+  additionalLayout->addRow(tr("Valid To:"), new QLabel(cert.expiryDate().toString(), this));
+  QStringList hostnames = cert.alternateSubjectNames().values(QSsl::DnsEntry);
+  for(int i = 0; i < hostnames.count(); i++) {
+    additionalLayout->addRow(tr("Hostname %1:").arg(i + 1), new QLabel(hostnames[i], this));
+  }
+  QStringList mailaddresses = cert.alternateSubjectNames().values(QSsl::EmailEntry);
+  for(int i = 0; i < mailaddresses.count(); i++) {
+    additionalLayout->addRow(tr("E-Mail Address %1:").arg(i + 1), new QLabel(mailaddresses[i], this));
+  }
+  additionalLayout->addRow(tr("Digest:"), new QLabel(QString(prettyDigest(cert.digest()))));
+  mainLayout->addWidget(additionalBox);
+
+
+  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok, Qt::Horizontal, this);
+  mainLayout->addWidget(buttonBox);
+
+  connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
+  connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
 }
+#endif
