@@ -126,7 +126,8 @@ MainWin::MainWin(QWidget *parent)
     sslLabel(new QLabel()),
     msgProcessorStatusWidget(new MsgProcessorStatusWidget()),
     _titleSetter(this),
-    _awayLog(0)
+    _awayLog(0),
+    _layoutLoaded(false)
 {
 #ifdef Q_WS_WIN
   dwTickCount = 0;
@@ -443,12 +444,11 @@ void MainWin::addBufferView(ClientBufferViewConfig *config) {
   BufferView *view = new BufferView(dock);
   view->setFilteredModel(Client::bufferModel(), config);
   view->installEventFilter(_inputWidget); // for key presses
-  view->show();
 
   Client::bufferModel()->synchronizeView(view);
 
   dock->setWidget(view);
-  dock->show();
+  dock->setVisible(_layoutLoaded); // don't show before state has been restored
 
   addDockWidget(Qt::LeftDockWidgetArea, dock);
   _bufferViewsMenu->addAction(dock->toggleViewAction());
@@ -468,6 +468,8 @@ void MainWin::removeBufferView(int bufferViewConfigId) {
     dock = qobject_cast<BufferViewDock *>(action->parent());
     if(dock && actionData.toInt() == bufferViewConfigId) {
       removeAction(action);
+      _bufferViews.removeAll(dock);
+      Client::bufferViewOverlay()->removeView(dock->bufferViewId());
       dock->deleteLater();
     }
   }
@@ -478,6 +480,11 @@ void MainWin::bufferViewToggled(bool enabled) {
   Q_ASSERT(action);
   BufferViewDock *dock = qobject_cast<BufferViewDock *>(action->parent());
   Q_ASSERT(dock);
+
+  // Make sure we don't toggle backlog fetch for a view we've already removed
+  if(!_bufferViews.contains(dock))
+    return;
+
   if(enabled) {
     Client::bufferViewOverlay()->addView(dock->bufferViewId());
     BufferViewConfig *config = dock->config();
@@ -740,7 +747,16 @@ void MainWin::setConnectedState() {
 void MainWin::loadLayout() {
   QtUiSettings s;
   int accountId = Client::currentCoreAccount().toInt();
-  restoreState(s.value(QString("MainWinState-%1").arg(accountId)).toByteArray(), accountId);
+  QByteArray state = s.value(QString("MainWinState-%1").arg(accountId)).toByteArray();
+  if(state.isEmpty()) {
+    // Make sure that the default bufferview is shown
+    if(_bufferViews.count())
+      _bufferViews.at(0)->show();
+    return;
+  }
+
+  restoreState(state, accountId);
+  _layoutLoaded = true;
 }
 
 void MainWin::saveLayout() {
@@ -761,6 +777,8 @@ void MainWin::updateLagIndicator(int lag) {
 void MainWin::disconnectedFromCore() {
   // save core specific layout and remove bufferviews;
   saveLayout();
+  _layoutLoaded = false;
+
   QVariant actionData;
   BufferViewDock *dock;
   foreach(QAction *action, _bufferViewsMenu->actions()) {
@@ -771,9 +789,12 @@ void MainWin::disconnectedFromCore() {
     dock = qobject_cast<BufferViewDock *>(action->parent());
     if(dock && actionData.toInt() != -1) {
       removeAction(action);
+      _bufferViews.removeAll(dock);
+      Client::bufferViewOverlay()->removeView(dock->bufferViewId());
       dock->deleteLater();
     }
   }
+
   QtUiSettings s;
   restoreState(s.value("MainWinState").toByteArray());
   setDisconnectedState();
