@@ -20,31 +20,24 @@
 
 #include "appearancesettingspage.h"
 
-#include "buffersettings.h"
-#include "chatviewsettings.h"
 #include "qtui.h"
 #include "qtuisettings.h"
 #include "qtuistyle.h"
-#include "util.h"
 
-#include <QDir>
-#include <QFontDialog>
-#include <QSignalMapper>
+#include <QCheckBox>
+#include <QFileDialog>
 #include <QStyleFactory>
+#include <QFile>
+#include <QDir>
 
 AppearanceSettingsPage::AppearanceSettingsPage(QWidget *parent)
-  : SettingsPage(tr("Appearance"), QString(), parent),
-  _fontsChanged(false)
+  : SettingsPage(tr("Interface"), QString(), parent)
 {
   ui.setupUi(this);
+  initAutoWidgets();
   initStyleComboBox();
   initLanguageComboBox();
 
-#ifndef HAVE_WEBKIT
-  ui.showWebPreview->hide();
-  ui.showWebPreview->setEnabled(false);
-#endif
-
   foreach(QComboBox *comboBox, findChildren<QComboBox *>()) {
     connect(comboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(widgetHasChanged()));
   }
@@ -52,16 +45,7 @@ AppearanceSettingsPage::AppearanceSettingsPage(QWidget *parent)
     connect(checkBox, SIGNAL(clicked()), this, SLOT(widgetHasChanged()));
   }
 
-  mapper = new QSignalMapper(this);
-  connect(mapper, SIGNAL(mapped(QWidget *)), this, SLOT(chooseFont(QWidget *)));
-
-  connect(ui.chooseChatView, SIGNAL(clicked()), mapper, SLOT(map()));
-  connect(ui.chooseBufferView, SIGNAL(clicked()), mapper, SLOT(map()));
-  connect(ui.chooseInputLine, SIGNAL(clicked()), mapper, SLOT(map()));
-
-  mapper->setMapping(ui.chooseChatView, ui.demoChatView);
-  mapper->setMapping(ui.chooseBufferView, ui.demoBufferView);
-  mapper->setMapping(ui.chooseInputLine, ui.demoInputLine);
+  connect(ui.chooseStyleSheet, SIGNAL(clicked()), SLOT(chooseStyleSheet()));
 }
 
 void AppearanceSettingsPage::initStyleComboBox() {
@@ -87,12 +71,7 @@ void AppearanceSettingsPage::initLanguageComboBox() {
 void AppearanceSettingsPage::defaults() {
   ui.styleComboBox->setCurrentIndex(0);
 
-  loadFonts(Settings::Default);
-  _fontsChanged = true;
-
-  ui.showWebPreview->setChecked(true);
-  ui.showUserStateIcons->setChecked(true);
-
+  SettingsPage::defaults();
   widgetHasChanged();
 }
 
@@ -120,36 +99,10 @@ void AppearanceSettingsPage::load() {
   ui.languageComboBox->setProperty("storedValue", ui.languageComboBox->currentIndex());
   Quassel::loadTranslation(selectedLocale());
 
-  ChatViewSettings chatViewSettings;
-  SettingsPage::load(ui.showWebPreview, chatViewSettings.showWebPreview());
-
-  BufferSettings bufferSettings;
-  SettingsPage::load(ui.showUserStateIcons, bufferSettings.showUserStateIcons());
-
-  loadFonts(Settings::Custom);
-
+  SettingsPage::load();
   setChangedState(false);
 }
 
-void AppearanceSettingsPage::loadFonts(Settings::Mode mode) {
-  QtUiStyleSettings s("Fonts");
-
-  QFont inputLineFont;
-  if(mode == Settings::Custom)
-    inputLineFont = s.value("InputLine", QFont()).value<QFont>();
-  setFont(ui.demoInputLine, inputLineFont);
-
-  QFont bufferViewFont;
-  if(mode == Settings::Custom)
-    bufferViewFont = s.value("BufferView", QFont()).value<QFont>();
-  setFont(ui.demoBufferView, bufferViewFont);
-
-  QTextCharFormat chatFormat = QtUi::style()->format(UiStyle::None, mode);
-  setFont(ui.demoChatView, chatFormat.font());
-
-  _fontsChanged = false;
-}
-
 void AppearanceSettingsPage::save() {
   QtUiSettings uiSettings;
 
@@ -165,32 +118,14 @@ void AppearanceSettingsPage::save() {
     uiSettings.setValue("Locale", selectedLocale());
   }
 
-  ChatViewSettings chatViewSettings;
-  chatViewSettings.enableWebPreview(ui.showWebPreview->isChecked());
+  bool needsStyleReload =
+        ui.useCustomStyleSheet->isChecked() != ui.useCustomStyleSheet->property("storedValue").toBool()
+    || (ui.useCustomStyleSheet->isChecked() && ui.customStyleSheetPath->text() != ui.customStyleSheetPath->property("storedValue").toString());
 
-  BufferSettings bufferSettings;
-  bufferSettings.enableUserStateIcons(ui.showUserStateIcons->isChecked());
-
-  // Fonts
-  QtUiStyleSettings fontSettings("Fonts");
-  if(ui.demoInputLine->font() != QApplication::font())
-    fontSettings.setValue("InputLine", ui.demoInputLine->font());
-  else
-    fontSettings.setValue("InputLine", "");
-
-  if(ui.demoBufferView->font() != QApplication::font())
-    fontSettings.setValue("BufferView", ui.demoBufferView->font());
-  else
-    fontSettings.setValue("BufferView", "");
-
-  QTextCharFormat chatFormat = QtUi::style()->format(UiStyle::None);
-  chatFormat.setFont(ui.demoChatView->font());
-  QtUi::style()->setFormat(UiStyle::None, chatFormat, Settings::Custom);
-
-  _fontsChanged = false;
-
-  load();
+  SettingsPage::save();
   setChangedState(false);
+  if(needsStyleReload)
+    QtUi::style()->reload();
 }
 
 QLocale AppearanceSettingsPage::selectedLocale() const {
@@ -206,24 +141,16 @@ QLocale AppearanceSettingsPage::selectedLocale() const {
   return locale;
 }
 
-void AppearanceSettingsPage::setFont(QLabel *label, const QFont &font_) {
-  QFont font = font_;
-  if(font.family().isEmpty())
-    font = QApplication::font();
-  label->setFont(font);
-  label->setText(QString("%1 %2").arg(font.family()).arg(font.pointSize()));
-  widgetHasChanged();
-}
+void AppearanceSettingsPage::chooseStyleSheet() {
+  QString dir = ui.customStyleSheetPath->property("storedValue").toString();
+  if(!dir.isEmpty() && QFile(dir).exists())
+    dir = QDir(dir).absolutePath();
+  else
+    dir = QDir(Quassel::findDataFilePath("default.qss")).absolutePath();
 
-void AppearanceSettingsPage::chooseFont(QWidget *widget) {
-  QLabel *label = qobject_cast<QLabel *>(widget);
-  Q_ASSERT(label);
-  bool ok;
-  QFont font = QFontDialog::getFont(&ok, label->font());
-  if(ok) {
-    setFont(label, font);
-    _fontsChanged = true;
-  }
+  QString name = QFileDialog::getOpenFileName(this, tr("Please choose a stylesheet file"), dir, "*.qss");
+  if(!name.isEmpty())
+    ui.customStyleSheetPath->setText(name);
 }
 
 void AppearanceSettingsPage::widgetHasChanged() {
@@ -231,14 +158,9 @@ void AppearanceSettingsPage::widgetHasChanged() {
 }
 
 bool AppearanceSettingsPage::testHasChanged() {
-  if(_fontsChanged) return true; // comparisons are nasty for now
-
   if(ui.styleComboBox->currentIndex() != ui.styleComboBox->property("storedValue").toInt()) return true;
 
   if(selectedLocale() != QLocale()) return true; // QLocale() returns the default locale (manipulated via loadTranslation())
 
-  if(SettingsPage::hasChanged(ui.showWebPreview)) return true;
-  if(SettingsPage::hasChanged(ui.showUserStateIcons)) return true;
-
   return false;
 }
