@@ -20,23 +20,33 @@
 
 #include "topicwidget.h"
 
-#include <QDebug>
-
 #include "client.h"
 #include "iconloader.h"
 #include "networkmodel.h"
+#include "uisettings.h"
 
 TopicWidget::TopicWidget(QWidget *parent)
   : AbstractItemView(parent)
 {
   ui.setupUi(this);
-  ui.topicEditButton->setPixmap(BarIcon("edit-rename"));
-
-  ui.topicLineEdit->hide();
+  ui.topicEditButton->setIcon(SmallIcon("edit-rename"));
+  ui.topicLineEdit->setWordWrapEnabled(true);
   ui.topicLineEdit->installEventFilter(this);
-  ui.topicLabel->show();
-  setContentsMargins(0,0,0,0);
-  parent->setMinimumHeight(layout()->sizeHint().height() + 2*qApp->style()->pixelMetric(QStyle::PM_DockWidgetTitleBarButtonMargin));
+
+  connect(ui.topicLabel, SIGNAL(clickableActivated(Clickable)), SLOT(clickableActivated(Clickable)));
+  connect(ui.topicLineEdit, SIGNAL(noTextEntered()), SLOT(on_topicLineEdit_textEntered()));
+
+  UiSettings s("TopicWidget");
+  s.notify("DynamicResize", this, SLOT(updateResizeMode()));
+  s.notify("ResizeOnHover", this, SLOT(updateResizeMode()));
+  updateResizeMode();
+
+  UiStyleSettings fs("Fonts");
+  fs.notify("UseCustomTopicWidgetFont", this, SLOT(setUseCustomFont(QVariant)));
+  fs.notify("TopicWidget", this, SLOT(setCustomFont(QVariant)));
+  if(fs.value("UseCustomTopicWidgetFont", false).toBool())
+    setCustomFont(fs.value("TopicWidget", QFont()));
+
 }
 
 void TopicWidget::currentChanged(const QModelIndex &current, const QModelIndex &previous) {
@@ -51,6 +61,31 @@ void TopicWidget::dataChanged(const QModelIndex &topLeft, const QModelIndex &bot
     setTopic(currentTopicIndex.data().toString());
 };
 
+void TopicWidget::setUseCustomFont(const QVariant &v) {
+  if(v.toBool()) {
+    UiStyleSettings fs("Fonts");
+    setCustomFont(fs.value("TopicWidget").value<QFont>());
+  } else
+    setCustomFont(QFont());
+}
+
+void TopicWidget::setCustomFont(const QVariant &v) {
+  UiStyleSettings fs("Fonts");
+  if(!fs.value("UseCustomTopicWidgetFont", false).toBool())
+    return;
+
+  setCustomFont(v.value<QFont>());
+}
+
+void TopicWidget::setCustomFont(const QFont &f) {
+  QFont font = f;
+  if(font.family().isEmpty())
+    font = QApplication::font();
+
+  ui.topicLineEdit->setCustomFont(font);
+  ui.topicLabel->setCustomFont(font);
+}
+
 void TopicWidget::setTopic(const QString &newtopic) {
   if(_topic == newtopic)
     return;
@@ -61,7 +96,25 @@ void TopicWidget::setTopic(const QString &newtopic) {
   switchPlain();
 }
 
-void TopicWidget::on_topicLineEdit_returnPressed() {
+void TopicWidget::updateResizeMode() {
+  StyledLabel::ResizeMode mode = StyledLabel::NoResize;
+  UiSettings s("TopicWidget");
+  if(s.value("DynamicResize", true).toBool()) {
+    if(s.value("ResizeOnHover", true).toBool())
+      mode = StyledLabel::ResizeOnHover;
+    else
+      mode = StyledLabel::DynamicResize;
+  }
+
+  ui.topicLabel->setResizeMode(mode);
+}
+
+void TopicWidget::clickableActivated(const Clickable &click) {
+  NetworkId networkId = selectionModel()->currentIndex().data(NetworkModel::NetworkIdRole).value<NetworkId>();
+  click.activate(networkId, _topic);
+}
+
+void TopicWidget::on_topicLineEdit_textEntered() {
   QModelIndex currentIdx = currentIndex();
   if(currentIdx.isValid() && currentIdx.data(NetworkModel::BufferTypeRole) == BufferInfo::ChannelBuffer) {
     BufferInfo bufferInfo = currentIdx.data(NetworkModel::BufferInfoRole).value<BufferInfo>();
@@ -78,26 +131,15 @@ void TopicWidget::on_topicEditButton_clicked() {
 }
 
 void TopicWidget::switchEditable() {
-  ui.topicLabel->hide();
-  ui.topicEditButton->hide();
-  ui.topicLineEdit->show();
+  ui.stackedWidget->setCurrentIndex(1);
   ui.topicLineEdit->setFocus();
-
-  setFixedHeight(layout()->sizeHint().height());
-  // Update the dock widget too, else it won't resize in all styles... FIXME try to sanitize this
-  qobject_cast<QWidget *>(parent())->setMinimumHeight(height() + 2*qApp->style()->pixelMetric(QStyle::PM_DockWidgetTitleBarButtonMargin));
-  qobject_cast<QWidget *>(parent())->adjustSize();
+  updateGeometry();
 }
 
 void TopicWidget::switchPlain() {
-  ui.topicLineEdit->hide();
-  ui.topicLabel->show();
-  ui.topicEditButton->show();
+  ui.stackedWidget->setCurrentIndex(0);
   ui.topicLineEdit->setText(_topic);
-  setFixedHeight(layout()->sizeHint().height());
-  // Update the dock widget too, else it won't resize in all styles... FIXME try to sanitize this
-  qobject_cast<QWidget *>(parent())->setMinimumHeight(height() + 2*qApp->style()->pixelMetric(QStyle::PM_DockWidgetTitleBarButtonMargin));
-  qobject_cast<QWidget *>(parent())->adjustSize();
+  updateGeometry();
 }
 
 // filter for the input widget to switch back to normal mode
